<?php

namespace Drupal\spammaster;

use Drupal\Core\Database\Connection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\State\StateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class service.
 */
class SpamMasterUpdaterService {

  /**
   * The database connection object.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The state.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The SpamMasterUpdaterService Service.
   *
   * @var \Drupal\spammaster\SpamMasterUpdaterService
   */
  protected $spamUpDb;

  /**
   * {@inheritdoc}
   */
  public function __construct(Connection $connection, ConfigFactoryInterface $configFactory, StateInterface $state) {
    $this->connection = $connection;
    $this->configFactory = $configFactory;
    $this->state = $state;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('config.factory'),
      $container->get('state')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function spamMasterUpDb($spamUpDb) {

    $spam_timenow = date('Y-m-d H:i:s');
    // Get db versions.
    $db_install_version_221 = $this->state->get('spammaster.db_install_version_221');

    // Process all legacy changes.
    if ('1' !== $db_install_version_221) {
      // State values, since each state is checked, can not use setmultiple.
      $spam_master_auto_update = $this->state->get('spammaster.spam_master_auto_update');
      if (empty($spam_master_auto_update)) {
        $this->state->set('spammaster.spam_master_auto_update', 'false');
      }
      $spam_master_db_protection_hash = $this->state->get('spammaster.spam_master_db_protection_hash');
      if (empty($spam_master_db_protection_hash)) {
        $spam_master_try_db_protection_hash = substr(md5(uniqid(mt_rand(), TRUE)), 0, 64);
        if (!empty($spam_master_try_db_protection_hash)) {
          $this->state->set('spammaster.spam_master_db_protection_hash', $spam_master_try_db_protection_hash);
        }
        else {
          $spam_master_again_db_protection_hash = 'md5-' . date('YmdHis');
          $this->state->set('spammaster.spam_master_db_protection_hash', $spam_master_again_db_protection_hash);
        }
      }
      $spam_master_license_sync_date = $this->state->get('spammaster.spam_master_license_sync_date');
      if (empty($spam_master_license_sync_date)) {
        $this->state->set('spammaster.spam_master_license_sync_date', date('Y-m-d'));
      }
      $spam_master_license_sync_run = $this->state->get('spammaster.spam_master_license_sync_run');
      if (empty($spam_master_license_sync_run)) {
        $this->state->set('spammaster.spam_master_license_sync_run', '0');
      }
      $spam_master_new_options = $this->state->get('spammaster.new_options');
      if (empty($spam_master_new_options)) {
        $this->state->set('spammaster.new_options', '0');
      }
      $spam_master_disc_not_date = $this->state->get('spammaster.spam_master_disc_not_date');
      if (empty($spam_master_disc_not_date)) {
        $this->state->set('spammaster.spam_master_disc_not_date', '1970-01-01');
      }
      $spam_master_disc_not = $this->state->get('spammaster.spam_master_disc_not');
      if (empty($spam_master_disc_not)) {
        $this->state->set('spammaster.spam_master_disc_not', '0');
      }
      $this->state->set('spammaster.white_transient', '0');
      // Delete state values.
      $delete_state_ar = [
        'spammaster.hourly_con',
        'spammaster.daily_con',
        'spammaster.weekly_con',
      ];
      $this->state->deleteMultiple($delete_state_ar);

      // Editable values, single checks entries and multi without entries check.
      $spammaster_settings_protection = $this->configFactory->getEditable('spammaster.settings_protection');
      $spam_master_is_cloudflare = $spammaster_settings_protection->get('spammaster.spam_master_is_cloudflare');
      if (empty($spam_master_is_cloudflare)) {
        $spammaster_settings_protection
          ->set('spammaster.spam_master_is_cloudflare', '0')
          ->save();
      }
      $spam_master_pre_signature = $spammaster_settings_protection->get('spammaster.signature');
      if (empty($spam_master_pre_signature)) {
        $spammaster_settings_protection
          ->set('spammaster.signature', '1')
          ->save();
      }
      $this->configFactory->getEditable('spammaster.settings_protection')
        ->set('spammaster.block_message', 'Your Email, Domain, or Ip are banned.')
        ->set('spammaster.basic_firewall', '1')
        ->set('spammaster.extra_honeypot', '1')
        ->set('spammaster.email_alert_3', '1')
        ->set('spammaster.email_daily_report', '0')
        ->set('spammaster.email_weekly_report', '0')
        ->set('spammaster.email_improve', '1')
        ->set('spammaster.cleanup_firewall', '2')
        ->set('spammaster.cleanup_honeypot', '2')
        ->set('spammaster.cleanup_whitelist', '2')
        ->set('spammaster.cleanup_system', '2')
        ->set('spammaster.cleanup_mail', '2')
        ->set('spammaster.cleanup_cron', '2')
        ->save();

      // Database legacy clean up and new inserts.
      $this->connection->delete('spammaster_keys')
        ->condition('spamkey', 'spammaster-registration', '=')
        ->execute();
      $this->connection->delete('spammaster_keys')
        ->condition('spamkey', 'spammaster-comment', '=')
        ->execute();
      $this->connection->delete('spammaster_keys')
        ->condition('spamkey', 'spammaster-contact', '=')
        ->execute();
      $this->connection->delete('spammaster_keys')
        ->condition('spamkey', 'spammaster-recaptcha', '=')
        ->execute();
      $this->connection->delete('spammaster_keys')
        ->condition('spamkey', 'spammaster-buffer', '=')
        ->execute();
      $this->connection->delete('spammaster_keys')
        ->condition('spamkey', 'spammaster-cleanup', '=')
        ->execute();
      $this->connection->delete('spammaster_keys')
        ->condition('spamkey', 'SPAMMASTER-DB-VERSION', '=')
        ->execute();
      // Check if exempt keys exist.
      $spammaster_exempt = $this->connection->query("SELECT spamvalue FROM {spammaster_keys} WHERE spamkey = :key AND spamvalue = :value", [
        ':key' => 'exempt-needle',
        ':value' => 'search',
      ]);
      $spammaster_exempt_result = $spammaster_exempt->fetchField();
      if (empty($spammaster_exempt_result)) {
        $spammaster_values = [
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'blockshop',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'cart',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'color_calc_form',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'commerce_checkout_flow_multistep_default',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'commerce_wishlist',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'content_moderation_entity',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'editor_image_dialog',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'editor_link_dialog',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'entity_view_display_layout_builder',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'lang_dropdown_form',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'mailchimp',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'mailchimp_signup_subscribe_block_sign_up_form',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'media_library_add_form_oembed',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'puppy_calc_form',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'quiz',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'rabbit_calc_form',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'rate',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'search',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'simplenews_subscri',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'shop',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'tfa_entry_form',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-needle',
            'spamvalue' => 'views_',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-key',
            'spamvalue' => 'ajax_page_state',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-key',
            'spamvalue' => 'delete-order-item',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-key',
            'spamvalue' => 'edit_quantity',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-key',
            'spamvalue' => 'editor_object',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-key',
            'spamvalue' => 'lang_dropdown_select',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-key',
            'spamvalue' => 'purchased_entity',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-key',
            'spamvalue' => 'search',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'Add+to+cart',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'Add to cart',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'Add to wishlist',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'Aggiungi al carrello',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'Ajouter au panier',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'angebot',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'color_calc_form',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'commerce_wishlist',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'courses-',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'Pay+and+complete+purchase',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'Pay and complete purchase',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'Proceed to checkout',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'puppy_calc_form',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'rabbit_calc_form',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'search',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'simplenews_subscri',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'tfa_entry_form',
          ],
          [
            'date' => $spam_timenow,
            'spamkey' => 'exempt-value',
            'spamvalue' => 'views_',
          ],
        ];
        $spammaster_query = $this->connection->insert('spammaster_keys')->fields(
          [
            'date',
            'spamkey',
            'spamvalue',
          ]
        );
        foreach ($spammaster_values as $exempt) {
          $spammaster_query->values($exempt);
        }
        $spammaster_query->execute();
      }
      // Updated the db version.
      $spdbversion = '221';
      // Update installed version.
      $this->state->set('spammaster.db_install_version', $spdbversion);
      // Set updater 221 as run 1.
      $db_install_version_221 = $this->state->set('spammaster.db_install_version_221', '1');
      // Log db update.
      $this->connection->insert('spammaster_keys')->fields([
        'date' => $spam_timenow,
        'spamkey' => 'spammaster',
        'spamvalue' => 'Spam Master: Updater service -> Success, updated db version to: ' . $spdbversion,
      ])->execute();
    }
  }

}
